#include "map.h"
//#include "../system/global.h"


#include <allegro.h>




CMap::CMap(void):dig_romb(NULL),color_mask(NULL),imgs_2d(NULL) {
	Logger::Log("\nCreate Map <object>\n");
	size_i = 0;
	size_j = 0;
	terra = new CImgLib();
	UnSelect();
	romb = CTerra::Romb(TERRA_SX,TERRA_SY,makecol(50,50,255));
}

CMap::~CMap() {
	Logger::Log("\nDestroy Map <object>\n");
	destroy_bitmap(romb);
	delete terra;
	if(dig_romb!=NULL) delete dig_romb;
	if(color_mask!=NULL) delete color_mask;
	printf("Destroy img library!\n");
	if(imgs_2d!=NULL) delete imgs_2d;
}

int CMap::Load(char *filename) 
{
	Logger::Log("Load map...\n");
	char *pathname = new char[255];
	char *imglist = new char[255];
	
	if(imgs_2d!=NULL) delete imgs_2d;
	imgs_2d = new CImgLib();
	
	int c=0;
	int i;
	for(i=strlen(filename)-1;i>=0;i--) 
		if(filename[i]=='/') break;
	strcpy(pathname,filename);
	pathname[i+1]='\0';

	printf("--->path to map: %s\n",pathname);
	strcpy(imglist,pathname);
	strcat(imglist,"images.list");
	printf("--->path to img. list: %s\n",imglist);
	imgs_2d->Load(imglist);

	
	MAPHeader *head = new MAPHeader();
	FILE *f = fopen(filename,"rb");
	if(f==NULL) {
		printf("\nERROR: can't open file: %s\n",filename);	
		allegro_exit();
		exit(10);
	}
	
	fread(head,sizeof(MAPHeader),1,f);
	size_i = head->size_i;
	size_j = head->size_j;
	printf("--->Set map size: %dx%d\n",size_i,size_j);	

	SetName(head->name);

	Logger::Log("--->Load map data...");
	for(int i=0;i<size_i;i++)
		for(int j=0;j<size_j;j++) {
			fread(&m_map[i][j],sizeof(MAP),1,f);
	}
	Logger::Log("OK\n");
	fclose(f);


	
	SetCoords();
	
	//setup_lightmap(imgs,8,18);
	//setup_lightmap(imgs,9,22);
	
	//setup_lightmap(imgs,22,16);
	    


	delete[] pathname;
	delete[] imglist;
	delete head;
	

	Logger::Log("OK\n");
	
}


void CMap::Finalize()
{
	printf("Finalize init map...\n");
	if(imgs_2d==NULL) {
		printf("ERROR: the img library is empty!\n");
		allegro_exit();
		exit(61); 
	}
	for(i=0;i<imgs_2d->GetSize();i++) {
		terra->Add(CTerra::Image(imgs_2d->Get(i)),imgs_2d->GetName(i));
	}
	if(dig_romb==NULL) dig_romb = CTerra::GetDigRomb(terra->Get(0));
	if(color_mask==NULL) color_mask = CTerra::create4color_mask(terra->Get(0));

	printf("Image library have %d entries!\n",terra->GetSize());
	delete imgs_2d;
	printf("Clear img library!\n");
	imgs_2d = NULL;
}


int CMap::Save(char *filename) 
{
	//Logger::Log("Save map...\n");
	if(terra == NULL) {
		printf("\nERROR: can't save unbuild map!\n");
		allegro_exit();
		exit(7);
	}
	FILE *f = fopen(filename,"wb");
	
	//Logger::Log("--->Create header...");
	MAPHeader *head = new MAPHeader();
	if(head == NULL) {
		printf("\nERROR: can't allocate memory for MAP header!\n");
		allegro_exit();
		exit(8);
	}
	//Logger::Log("OK\n");
	strcpy(head->name,"Test map");
	head->version = 1;
	head->size_i = size_i;
	head->size_j = size_j;

	head->textures = terra->GetSize();
	//Logger::Log("--->Write header...");
	fwrite(head,sizeof(MAPHeader),1,f);
	//Logger::Log("OK\n");
	

	//char texture[32];
	//Logger::Log("--->Write textures name");
	//for(int i=0;i<head->textures;i++) {
	//	strcpy(texture,terra->GetName(i));
	//	fwrite(texture,32,1,f);
	//}
	//Logger::Log("OK\n");
	for(int i=0;i<size_i;i++)
		for(int j=0;j<size_j;j++) {
		fwrite(&m_map[i][j],sizeof(MAP),1,f);
	}
	delete head;
	fclose(f);
	return 0;
}



int CMap::Live() {}



//--------------------------------------------------------------
int CMap::Show() 
{
	int i,j;
	for(i=0;i<size_i;i++) 
		for(j=0;j<size_j;j++) {
			if((m_map[i][j].x+map_x>-TERRA_SX && m_map[i][j].x+map_x<640) &&
			   (m_map[i][j].y+map_y>-TERRA_SY && m_map[i][j].y+map_y<480)) {
				CTerra::draw_dig_romb(p_screen,terra->Get(m_map[i][j].type),dig_romb,
										m_map[i][j].x+map_x,m_map[i][j].y+map_y);
				if(m_map[i][j].selected) draw_sprite(p_screen,romb,m_map[i][j].x+map_x,m_map[i][j].y+map_y);
			}
	}
                                                                
}

//--------------------------------------------------------------
void CMap::Move(int x,int y) 
{
	map_x += x;
	map_y += y;
}

//--------------------------------------------------------------
void CMap::SetXY(int x,int y) 
{
	this->map_x = x;
	this->map_y = y;
}

void CMap::GetXY(int* x, int* y)
{
	*x = this->map_x;
	*y = this->map_y;
}

int CMap::GetZ()
{
	return -1; 
}

//--------------------------------------------------------------


void CMap::GenerateTextures(char *filename,char *pc_name)
{
	printf("\nAdd texture: %s\n",pc_name);
	terra->Add(CTerra::Image(filename),pc_name);
	if(dig_romb==NULL) dig_romb = CTerra::GetDigRomb(terra->Get(0));
}


void CMap::TestInit()
{
	GenerateTextures("data/textures/grass02.bmp","grass0");      // - 0
	GenerateTextures("data/textures/grass00.bmp","grass1");      // - 1
	GenerateTextures("data/textures/grass02.bmp","grass2");      // - 2
	GenerateTextures("data/textures/grass00.bmp","grass3");      // - 3
	GenerateTextures("data/textures/grass01.bmp","grass4");      // - 4
	
	GenerateTextures("data/textures/roadup.bmp","road up0");   // - 5
	
	GenerateTextures("data/textures/stone01.bmp","stones0");    // - 6
	GenerateTextures("data/textures/stone02.bmp","stones1");    // - 7

	GenerateTextures("data/textures/grass-stone01.bmp","grass with stones0"); // -8
	GenerateTextures("data/textures/grass-stone11.bmp","grass with stones1"); // -9
	GenerateTextures("data/textures/grass-stone21.bmp","grass with stones2"); // 10
    GenerateTextures("data/textures/grass-stone31.bmp","grass with stones3"); // 11


	for(int i=0;i<100;i++)
		for(int j=0;j<100;j++) m_map[i][j].type = rand()%5;


	for(int j=0;j<50;j++) {
		m_map[1][j].type = 5;
		m_map[3][j].type = 5;
	}

	for(int i=3;i<6;i++) {
		m_map[i][6].type = 8;
		m_map[2][i].type = 9;
		m_map[i][2].type = 11;	
	}
	

	for(int i=3;i<6;i++)
		for(int j=3;j<6;j++) m_map[i][j].type = rand()%2+6;
	
	m_map[2][6].type = 10;

	size_i=100;
	size_j=100;

	map_x=0;
	map_y=0;
	
	SetName("MAP");

}


void CMap::UnSelect()
{
	//sel_i = -1;
	//sel_j = -1;
	int i,j;
	for(i=0;i<size_i;i++)
		for(j=0;j<size_j;j++) m_map[i][j].selected = false; 	
}


void CMap::Select(int i,int j)
{
	//sel_i = i;
	//sel_j = j;
	if(i<0 || j<0) return;
	m_map[i][j].selected = true;
}


void CMap::GetMapCoord(int px,int py,int *pi,int *pj)
{
	register int x,y;
	int i,j;
	int color;

	if(color_mask==NULL) {
		printf("\nERROR: color mask nust be set!\n");
		return;
	}

	for(i=0;i<size_i;i++)
		for(j=0;j<size_j;j++) {
			x = m_map[i][j].x + map_x;
			y = m_map[i][j].y + map_y;
			if((px>=x && px<x+TERRA_SX) && 
			   (py>=y && py<y+TERRA_SY)) {
			    
				color = getpixel(color_mask,px-x,py-y);
				
				if(color==makecol(0,0,0)) {
					*pi = i;
					*pj = j;
					return;
				} else
				if(color==makecol(255,0,0)) {
					*pi = i;
					*pj = j-1;
					return;
				} else
				if(color==makecol(0,255,0)) {
					*pi = i+1;
					*pj = j;
					return;
				} else
				if(color==makecol(0,0,255)) {
					*pi = i-1;
					*pj = j;
					return; 
				} else {
					*pi = i;
					*pj = j+1;
					return;
				}
			}
	}

	*pi=-1;
}




char* CMap::GetTerraName(int i,int j)
{
	if(i<0 || j<0 || i>size_i-1 || j>size_j-1 || m_map[i][j].type==-1) return "empty";

	return terra->GetName(m_map[i][j].type);
	
}


void CMap::SetCoords()
{
	int i,j;
	for(i=0;i<size_i;i++)
		for(j=0;j<size_j;j++) {
			m_map[i][j].x = 0 + i*TERRA_SX_2+j*TERRA_SX_2;
			m_map[i][j].y = 0 + j*TERRA_SY_2-i*TERRA_SY_2;
	}
}

void CMap::GetSizeIJ(int *pi_size_i,int *pi_size_j)
{
	*pi_size_i = size_i;
	*pi_size_j = size_j;
}

void CMap::GetSizes(int *psx, int *psy)
{
	*psx = size_i;
	*psy = size_j;
}

bool CMap::GetFreeStatus(int i,int j)
{
	return m_map[i][j].is_free;
}

void CMap::SetFreeStatus(int i,int j,bool stat)
{
	m_map[i][j].is_free = stat;
}

void CMap::setup_lightmap(int li,int lj)
{
    
	int starti=li-3,startj=lj-3;
	
	Logger::Log("SetUp light maps...");	
	if(imgs_2d==NULL) {
		printf("ERROR: the img library is empty!\n");
		allegro_exit();
		exit(6);
	}

	int lx = 0 + li*TERRA_SX_2+lj*TERRA_SX_2;
	int ly = 0 + lj*TERRA_SY_2-li*TERRA_SY_2;
	BITMAP *lt;
	BITMAP *ter;
	int img_num;
	int i,j;
	char c_name[356];
	int match=0;
	for(i=starti;i<=starti+7;i++)
		for(j=startj;j<=startj+7;j++) {
			if(i<0 || j<0) continue;
			
            //strcpy(c_name,imgs_2d->GetName(m_map[i][j].type));
			//strcat(c_name,"-light");
			//printf("Make name for type: %d\n",m_map[i][j].type);
			sprintf(c_name,"%s-light-%d:%d",imgs_2d->GetName(m_map[i][j].type),
											i-starti,j-startj);
			
			//printf("search in library for name: %s\n",c_name);
			//search in library
			img_num = imgs_2d->GetNum(c_name);
			if(img_num!=-1) {
				m_map[i][j].type = img_num;
				//printf("Match found: %d\n",img_num);
				match++;
				continue;
			}

			ter = imgs_2d->Get(m_map[i][j].type);
			if(ter==NULL) {
				printf("ERROR: can't find texture!\n");
				continue;
			}
			
			//draw_sprite(screen,ter,10,10);
			//printf("===[i:%d,j:%d]> tx: %d   ty: %d\n",i,j,m_map[i][j].x,m_map[i][j].y);
			lt = CTerra::make_light_texture(ter,
				0,0,0,10,
				(i-starti)*64,(j-startj)*64,
				4*64,4*64);
			img_num = imgs_2d->Add(lt,c_name);
			//printf("Add to library name: %s:%d\n",c_name,img_num);
			
			//printf("OK!\n===[i:%d,j:%d]>Add to m_map...",i+starti,j+startj);
			m_map[i][j].type = img_num;
			//printf("OK!\n");
	}
	Logger::Log("OK!");
	printf("  %d matches found!\n",match);	
	
}

