/*
	OBJECT
	class work with object. As a houses and etc.

	A-Games(C)Igor Rudym
	BetaWorld/Oasis Project
*/

#include "object.h"
#include "../system/log.h"
#include "../map/map.h"

IObject* gp_cur_object = NULL;

Object::Object():luaScript()
{
	init_class();	
}

Object::Object(char* filename):luaScript()
{
	init_class();
	Load(filename);
}

void Object::init_class()
{
	current_image = 0;
	map = NULL;
	selected = false;
	init_script_functions();
}

Object::~Object()
{
	char buffer[255];
	sprintf(buffer,"Destroy object: %s\n",name);
	Logger::Log(buffer);
	
	for(int i=tasks.size()-1;i>=0;i--) {
		delete tasks.front();
		tasks.pop();
	}
}

int Object::Load(char *filename)
{
	//ObjectScript* loader = new ObjectScript(filename,this);
	//loader->parser();
	//loader->Run();
	//delete loader;
}

int Object::Save(char *filename)
{
}

int Object::Live()
{
	//printf("Object[%s]->Live()\n",name);
	gp_cur_object = this;
	lua_getglobal(lp_L,"live");
	lua_call(lp_L,0,0);
}

int Object::Show()
{
	int sx,sy;
	register BITMAP *bmp = images->Get(current_image,&sx,&sy);
	//draw_sprite(p_screen,bmp,x+sx,y+sy);
	
	int cx1,cy1,cx2,cy2;
	cx1 = p_screen->cl;
	cy1 = p_screen->ct;
	cx2 = p_screen->cr;
	cy2 = p_screen->cb;
	
	int ox = 0;
	int oy = 0;
	int osx = bmp->w;
	int osy = bmp->h;
	int dx = x+sx;
	int dy = y+sy;	

	if(dx<cx1) {
		ox = cx1 - dx;
		osx-=(cx1-dx);
		dx = cx1;
	}
	if(dy<cy1) {
		oy = cy1 - dy;
		osy-=(cy1-dy);
		dy = cy1;
	}
	if(dx>cx2-osx) osx = cx2-dx;
	if(dy>sy>cy2-osy) osy = cy2-dy;

	masked_blit(bmp,p_screen,ox,oy,dx,dy,osx,osy); 
	
	if(selected) {
		rect(p_screen,x+sx,y+sy,x+sx+bmp->w,y+sy+bmp->h,makecol(255,0,0));
	}
}


int Object::GetZ()
{
	return z;
}


void Object::SetIJ(int ni, int nj)
{
	i = ni;
	j = nj;
	SetUpCoords();
	for(int ii=i;ii<i+size_i;ii++)
		for(int ij=j;ij<j+size_j;ij++)
			((CMap*)map)->SetFreeStatus(ii,ij,false);
}
   
void Object::GetIJ(int *p_i, int *p_j)
{
	*p_i = i;
	*p_j = j;	
}


CImgLib* Object::GetImages()
{
	return images;
}

void Object::SetImages(CImgLib* imgs)
{
	images = imgs;	
}


void Object::SetUpCoords()
{
	int map_x,map_y;

	map->GetXY(&map_x,&map_y);

	x = 0 + i*TERRA_SX_2+j*TERRA_SX_2+map_x;
	y = 0 + j*TERRA_SY_2-i*TERRA_SY_2+map_y;
	//
	if(map==NULL) return;
	int size_i=32,size_j=32;
	//Logger::Log("Get map sizes...");
	((CMap*)map)->GetSizeIJ(&size_i,&size_j);
	//Logger::Log("OK\n");
	if(GetPart(size_i,size_j)==1) z = size_i-i;
		else z = size_i+j-1;
	z-=i;
	//printf("\nSetup coords of %s (%d,%d) | Z = %d\n",name,i,j,z);
}

void Object::SetMap(IObject *mp_map)
{
	map = mp_map;
}

int Object::GetPart(int size_i,int size_j)
{
	if(j<i) return 1;
		else return 2;
}

void Object::GetSizes(int *psx, int *psy)
{
	BITMAP *tmp = images->Get(current_image);
	*psx = tmp->w;
	*psy = tmp->h;
}

void Object::GetXY(int *ip_x,int *ip_y) {
	int shift_x,shift_y;
	images->GetShifts(current_image,&shift_x,&shift_y);
	*ip_x = x+shift_x;
	*ip_y = y+shift_y;
}

void Object::Select()
{
	selected = true;
}

void Object::UnSelect()
{
	selected = false;
}

bool Object::contains(int i_x,int i_y)
{
	register int ox,oy;
	register int sox,soy;
	GetXY(&ox,&oy);
	GetSizes(&sox,&soy);
	if((i_x>=ox && i_x<=ox+sox) && (i_y>=oy && i_y<=oy+soy)) {
		if(Tools::InBitmap(images->Get(current_image),i_x-ox,i_y-oy)) return true;
	}
	return false;
}

void Object::SetTask(STask* ptask)
{
	tasks.push(ptask);
}

STask* Object::GetTask()
{
	register STask* ptask = tasks.front();
	tasks.pop();
	return ptask;
}

STask* Object::ViewTask()
{
	return  tasks.front();
}

STask* Object::LastTask()
{
	return  tasks.back();
}







//---------------LUA
int Object::LoadScript(const char* filename)
{
	if(lua_dofile(lp_L,filename)==LUA_ERRFILE) {
		printf("ERROR: can't open file: %s\n",filename);
		allegro_exit();
		exit(87);
	}
}


void Object::init_script_functions()
{
	lua_register(lp_L,"set_ij",lua_set_ij);
	lua_register(lp_L,"set_flag",lua_set_flag);
	lua_register(lp_L,"free_place",lua_free_place);
	lua_register(lp_L,"set_current_image",lua_set_current_image);
	lua_register(lp_L,"prev_image",lua_prev_image);
	lua_register(lp_L,"next_image",lua_next_image);
}

int Object::lua_set_ij(lua_State *L)
{
	int n = lua_gettop(L);
	if(n<2) printf("Error! need more arguments!");
	((Object*)gp_cur_object)->SetIJ((int)lua_tonumber(L,1),(int)lua_tonumber(L,2));
}

int Object::lua_free_place(lua_State *L)
{
}


int Object::lua_set_flag(lua_State *L)
{
	int n = lua_gettop(L);
	((Object*)gp_cur_object)->SetFlag((int)lua_tonumber(L,1));
}


int Object::lua_set_current_image(lua_State *L)
{
	int n = lua_gettop(L);
	
	//printf("Set current image: %s\n",lua_tostring(L,1));
	((Object*)gp_cur_object)->SetCurrentImage((char*)lua_tostring(L,1));

	return 0;
}

int Object::lua_prev_image(lua_State* L)
{	
	int n = lua_gettop(L);
	int curi = ((Object*)gp_cur_object)->GetCurrentImage();
	curi--;
	if(curi<0) curi = ((Object*)gp_cur_object)->GetMaxImage();
	((Object*)gp_cur_object)->SetCurrentImage(curi);
}

int Object::lua_next_image(lua_State* L)
{
	int n = lua_gettop(L);
	int curi = ((Object*)gp_cur_object)->GetCurrentImage();
	curi++;
	if(curi>((Object*)gp_cur_object)->GetMaxImage()) curi = 0;
	((Object*)gp_cur_object)->SetCurrentImage(curi);
}
	





