#ifndef __OBJECT_H
#define __OBJECT_H

#include "../interfaces/iobject.h"
#include "common.h"
#include "../tools/imglib.h"
#include "../map/map.h"
#include "../lua-script/luascript.h"
#include "task.h"



class Object : public CommonObject, luaScript {
public:
	Object();
	Object(char* filename);
	virtual ~Object();

	void init_class();

	virtual int Load(char *filename);
	virtual int Save(char *filename);

	virtual int Live();

	virtual int Show();

	virtual int GetZ();


	virtual CImgLib* GetImages();
	virtual void SetImages(CImgLib*);

	virtual void GetXY(int *, int *);

    virtual void SetIJ(int i, int j);
    virtual void GetIJ(int *, int *);

    //   x,y  i,j
    virtual void SetUpCoords();

	virtual void SetMap(IObject *map);

	virtual void GetSizes(int *, int *);

	virtual void Select();
	virtual void UnSelect();

	virtual int GetType() {return OBJ_OBJECT;}

	//    (i_x,i_y)
	virtual bool contains(int i_x,int i_y);   


	virtual void SetCurrentImage(int img_num) {
		current_image = img_num;
	}

	virtual int GetCurrentImage() {
		return current_image;
	}
	
	virtual int GetMaxImage() {
		return images->GetSize()-1; 
	}

	virtual void SetCurrentImage(char *img_name) {
		int img_num = images->GetNum(img_name);
		if(img_num!=-1) SetCurrentImage(img_num);
	}

//--------------- Tasks
	virtual void SetTask(STask*);
	virtual STask* GetTask();
	virtual STask* ViewTask();
	virtual STask* LastTask();

//--------------- LUA
	
	virtual int LoadScript(const char* filename);
	virtual void init_script_functions();

private:
	static int lua_set_ij(lua_State *L);
	static int lua_free_place(lua_State *L);
	static int lua_set_flag(lua_State *L);
	static int lua_set_current_image(lua_State *L);
	static int lua_prev_image(lua_State* L);
	static int lua_next_image(lua_State* L);
	

protected:

	int GetPart(int size_i,int size_j);	


	int i,j;
	int z;
	CImgLib *images;
	int current_image;
	IObject *map;
	bool selected;
	char *image_lib_name;
	PTaskQueue tasks;
};






#endif