//SCENE for BetaWorld 1.8

#include "scene.h"
#include "../system/global.h"
#include "../objects/object.h"
#include "../objects/creature.h"
#include "../objects/objectloader.h"
#include "../map/map.h"


ImagesPtrArray *g_imgs_lib;
ObjectPtrArray *g_objects;


CScene::CScene():luaScript(),name(NULL) {
}
CScene::CScene(char *cp_name):luaScript() 
{
	SetName(cp_name);
}

CScene::~CScene() 
{
	for(int i=objects.size()-1;i>=0;i--) {	
		if(objects[i]!=NULL) delete objects[i];
		objects.pop_back();
	}
	for(int i=imgs_lib.size()-1;i>=0;i--) {
		delete imgs_lib[i];
		imgs_lib.pop_back();
	}
	if(name!=NULL) delete name;
}

int CScene::Load(char *filename) 
{
	init_script_function();
	lua_dofile(lp_L,filename);
	((CMap*)objects[0])->Finalize();
}


void CScene::init_script_function()
{
	lua_register(lp_L,"load_imglib",load_imglib);
	lua_register(lp_L,"load_object",load_object);
	lua_register(lp_L,"load_creature",load_creature);	
	lua_register(lp_L,"put_light",put_light);

	g_imgs_lib = &imgs_lib;
	g_objects = &objects;
}


int CScene::load_imglib(lua_State *L)
{
	int n = lua_gettop(L);
	
	CImgLib *imgs = new CImgLib();
	printf("Load library: %s...\n",lua_tostring(L,1));
	if(imgs->Load((char*)lua_tostring(L,1)) == ERROR_OPEN_FILE) {
		allegro_exit();
		printf("\nERROR: can't open images file: %s\n",lua_tostring(L,1));
		exit(13);
	}
	printf("OK!\nSet library name: %s\n",lua_tostring(L,2));
	
		imgs->SetLibName((char*)lua_tostring(L,2));
		g_imgs_lib->push_back(imgs);
		
}

int CScene::load_object(lua_State *L)
{
	int n = lua_gettop(L);

	CObjectLoader *loader = new CObjectLoader();

	char filename[255];
	strcpy(filename,"data/world/objects/");
	strcat(filename,(char*)lua_tostring(L,1));
	
	printf("==>Load object: %s\n",lua_tostring(L,2)); 
	IObject *obj = loader->Load(filename,"object",
								(char*)lua_tostring(L,2),*g_imgs_lib,
								(*g_objects)[0],
								(int)lua_tonumber(L,3),
								(int)lua_tonumber(L,4));
	printf("==>OK!\n");
	g_objects->push_back(obj);
	delete loader;
}


int CScene::load_creature(lua_State *L)
{
	int n = lua_gettop(L);

	CObjectLoader *loader = new CObjectLoader();

	char filename[255];
	strcpy(filename,"data/world/objects/");
	strcat(filename,(char*)lua_tostring(L,1));
	
	printf("==>Load creature: %s\n",lua_tostring(L,2)); 
	IObject *obj = loader->Load(filename,"creature",
								(char*)lua_tostring(L,2),*g_imgs_lib,
								(*g_objects)[0],
								(int)lua_tonumber(L,3),
								(int)lua_tonumber(L,4));
	printf("==>OK!\n");
	g_objects->push_back(obj);
	delete loader;
}




int CScene::put_light(lua_State *L)
{
	int n = lua_gettop(L);
	int i = (int)lua_tonumber(L,1);
	int j = (int)lua_tonumber(L,2);

	((CMap*)(*g_objects)[0])->setup_lightmap(i,j);
}



int CScene::Save(char *filename) {}


 
int CScene::Show()
{
	
	for(unsigned int i=0;i<OnScreen.size();i++) {
		//printf("SCENE::Show object: %s  | Z = %d\n",OnScreen[i]->GetName(),
		//									OnScreen[i]->GetZ());
		OnScreen[i]->Show();
		//printf("Show object: %s  | Z = %d\n",OnScreen[i]->GetName(),
		//									OnScreen[i]->GetZ());
	}
	return 1;
}
	
int CScene::Live() 
{
	//printf("CScene->Live()\n");
	for(unsigned int i=0;i<objects.size();i++)
		objects[i]->Live();
	return 1;
}

int CScene::Animation()
{
	for(unsigned int i=0;i<objects.size();i++)
		objects[i]->Animation();
	return 0; 
}

void CScene::Add(IObject *obj)
{
	if(obj==NULL) {
		printf("\nERROR: Can't add NULL object!");
		allegro_exit();
		exit(6);
	}
	//printf("SCENE::Add Object: %s ...\n",obj->GetName());
	//if(objects.size()>0) obj->SetObject(objects[0]);
	objects.push_back(obj);
	//printf("OK\n");
}


void CScene::SetScreen(BITMAP* scr)
{
	//buffer = scr;
    for(unsigned int i=0;i<objects.size();i++) objects[i]->SetScreen(scr);
}


void CScene::Move(int x,int y)
{
	//printf("SCENE::Move()\n");
	for(unsigned int i=0;i<objects.size();i++) objects[i]->Move(x,y);	
	SetUpObjectsOnScene();
	//printf("^-OK!\n");
}


void CScene::SetXY(int x,int y)
{
	for(unsigned int i=0;i<objects.size();i++) objects[i]->SetXY(x,y);
}

void CScene::GetXY(int*x, int*y)
{
	*x = -1;
	*y = -1;	
}

void CScene::SetName(char *cp_name)
{
	if(name!=NULL) delete name;
	name = new char[strlen(cp_name)+1];
	strcpy(name,cp_name);
}

char* CScene::GetName()
{
	return name;
}	
                          
void CScene::SortObjects()
{
	//printf("SCENE::SortObject()...");
	IObject *tmp;
	int i,j;
	if(OnScreen.size()<=1) return;
	for(i=1;i<OnScreen.size()-1;i++) 
		for(j=i+1;j<OnScreen.size();j++) {
			//printf("Sorting (i:%d,j:%d): Z1:%d <?> Z2:%d\n",i,j,
			//		OnScreen[i]->GetZ(),OnScreen[j]->GetZ());
			if(OnScreen[i]->GetZ() > OnScreen[j]->GetZ()) {
				//printf("--->SWAP!\n");
				tmp = OnScreen[i];
				OnScreen[i] = OnScreen[j];
				OnScreen[j] = tmp;
			}
	}
	//printf("OK!\n");
}
	
int CScene::GetObjectIndex(char *name)
{
	for(unsigned int i=1;i<objects.size();i++) {
		if(!strcmp(name,objects[i]->GetName())) return i;
	}
	return -1;
}


void CScene::SetUpObjectsOnScene()
{
	OnScreen.clear();
	int sx,sy;
	int x,y;
	OnScreen.push_back(objects[0]);
	for(unsigned int i=1;i<objects.size();i++) {
		//printf("Object: %s\n",objects[i]->GetName());
		if(((CommonObject*)objects[i])->GetFlag() & OBJ_NONVISIBLE)
			continue;	
		objects[i]->GetSizes(&sx,&sy);
		objects[i]->GetXY(&x,&y);
		if(((x+sx>0 && y+sy>0) && (x<GFX_SX && y<GFX_SY)) || 
			objects[i]->GetType()==OBJ_FX) {
			//printf("SCENE::Push to screen: %s...",objects[i]->GetName());
			OnScreen.push_back(objects[i]);
		    //printf("OK!\n");
		}
	}
	SortObjects();
}

void CScene::SelectObject(int mx,int my)
{
	if(OnScreen.size()==0) return;
	UnSelectObjects();
	for(unsigned int i=OnScreen.size()-1;i>0;i--) {
		if(((Object*)OnScreen[i])->GetFlag() & OBJ_NONSELECT) continue;
		if(((Object*)OnScreen[i])->contains(mx,my)) {
			((Object*)OnScreen[i])->Select();
			break;
		}
	}
}

void CScene::UnSelectObjects()
{
	if(OnScreen.size()==0) return;
	for(unsigned int i=1;i<OnScreen.size();i++) {
		if(((Object*)OnScreen[i])->GetFlag() & OBJ_NONSELECT) continue;
		((Object*)OnScreen[i])->UnSelect();
	}
}


IObject* CScene::Get(char *name)
{
	for(int i=0;i<objects.size();i++) {
		if(!strcmp(name,objects[i]->GetName())) return objects[i];
	}
	return NULL;
}

IObject* CScene::Get(int num)
{
	return objects[num];
}



                                                        